@@ -1,6 +1,8 @@
 import asyncio
 import collections
 
+import synapse.exc as s_exc
+import synapse.common as s_common
 import synapse.lib.base as s_base
 
 class AQueue(s_base.Base):
@@ -40,6 +42,75 @@ class AQueue(s_base.Base):
         self.event.clear()
         return retn
 
+class Queue:
+    '''
+    An asyncio Queue with batch methods and graceful close.
+    '''
+    def __init__(self, maxsize=None):
+        self.q = asyncio.Queue(maxsize=maxsize)
+        self.closed = False
+
+    async def close(self):
+        await self.q.put(s_common.novalu)
+        self.closed = True
+
+    async def put(self, item):
+
+        if self.closed:
+            mesg = 'The Queue has been closed.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        await self.q.put(item)
+
+    async def size(self):
+        size = self.q.qsize()
+        if self.closed:
+            size -= 1
+        return size
+
+    async def puts(self, items):
+
+        if self.closed:
+            mesg = 'The Queue has been closed.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        for item in items:
+            await self.q.put(item)
+
+    async def slice(self, size=1000):
+
+        if self.closed and self.q.qsize() == 0:
+            return None
+
+        items = []
+
+        item = await self.q.get()
+        if item is s_common.novalu:
+            return None
+
+        items.append(item)
+
+        size -= 1
+
+        for i in range(min(size, self.q.qsize())):
+
+            item = await self.q.get()
+            if item is s_common.novalu:
+                break
+
+            items.append(item)
+
+        return items
+
+    async def slices(self, size=1000):
+
+        while True:
+            items = await self.slice(size=size)
+            if items is None:
+                return
+
+            yield items
+
 class Window(s_base.Base):
     '''
     A Queue like object which yields added items.  If the queue ever reaches
