@@ -170,7 +170,7 @@ class Snap(s_base.Base):
                 yield node
 
     async def nodes(self, text, opts=None, user=None):
-        return [n async for n in self.eval(text, opts=opts, user=user)]
+        return [node async for (node, path) in self.storm(text, opts=opts, user=user)]
 
     async def clearCache(self):
         self.tagcache.clear()
@@ -511,6 +511,10 @@ class Snap(s_base.Base):
 
         async def _getadds(f, p, formnorm, forminfo, doaddnode=True):
 
+            if f.locked:
+                mesg = f'Form {f.full} is locked due to deprecation.'
+                raise s_exc.IsDeprLocked(mesg=mesg)
+
             edits = []  # Non-primary prop edits
             topsubedits = []  # Primary prop sub edits
 
@@ -525,8 +529,14 @@ class Snap(s_base.Base):
                 if prop is None:
                     continue
 
+                if prop.locked:
+                    mesg = f'Prop {prop.full} is locked due to deprecation.'
+                    raise s_exc.IsDeprLocked(mesg=mesg)
+
                 assert prop.type.stortype is not None
 
+                propnorm, typeinfo = prop.type.norm(propvalu)
+
                 if isinstance(prop.type, s_types.Ndef):
                     ndefname, ndefvalu = propvalu
                     ndefform = self.core.model.form(ndefname)
@@ -545,7 +555,7 @@ class Snap(s_base.Base):
                 elif isinstance(prop.type, s_types.Array):
                     arrayform = self.core.model.form(prop.type.arraytype.name)
                     if arrayform is not None:
-                        for arrayvalu in propvalu:
+                        for arrayvalu in propnorm:
                             arraynorm, arrayinfo = arrayform.type.norm(arrayvalu)
 
                             if self.buidprefetch:
@@ -555,8 +565,6 @@ class Snap(s_base.Base):
 
                             subedits.extend([x async for x in _getadds(arrayform, {}, arraynorm, arrayinfo)])
 
-                propnorm, typeinfo = prop.type.norm(propvalu)
-
                 propsubs = typeinfo.get('subs')
                 if propsubs is not None:
                     for subname, subvalu in propsubs.items():
@@ -602,6 +610,10 @@ class Snap(s_base.Base):
             else:
                 yield (buid, f.name, edits)
 
+        if self.core.maxnodes is not None and self.core.maxnodes <= self.core.nodecount:
+            mesg = f'Cortex is at node:count limit: {self.core.maxnodes}'
+            raise s_exc.HitLimit(mesg=mesg)
+
         if props is None:
             props = {}
 
